searchData={"items":[{"type":"module","title":"Flop.Phoenix","doc":"Phoenix components for pagination, sortable tables and filter forms with\n[Flop](https://hex.pm/packages/flop).","ref":"Flop.Phoenix.html"},{"type":"module","title":"Introduction - Flop.Phoenix","doc":"Please refer to the [Readme](README.md) for an introduction.","ref":"Flop.Phoenix.html#module-introduction"},{"type":"module","title":"Customization - Flop.Phoenix","doc":"The default classes, attributes, texts and symbols can be overridden by\npassing the `opts` assign. Since you probably will use the same `opts` in all\nyour templates, you can globally configure an `opts` provider function for\neach component.\n\nThe functions have to return the options as a keyword list. The overrides\nare deep-merged into the default options.\n\n    defmodule MyAppWeb.CoreComponents do\n      import Phoenix.HTML.Tag\n\n      def pagination_opts do\n         [\n          ellipsis_attrs: [class: \"ellipsis\"],\n          ellipsis_content: \"‥\",\n          next_link_attrs: [class: \"next\"],\n          next_link_content: next_icon(),\n          page_links: {:ellipsis, 7},\n          pagination_link_aria_label: &\"#{&1}ページ目へ\",\n          previous_link_attrs: [class: \"prev\"],\n          previous_link_content: previous_icon()\n        ]\n      end\n\n      defp next_icon do\n        tag :i, class: \"fas fa-chevron-right\"\n      end\n\n      defp previous_icon do\n        tag :i, class: \"fas fa-chevron-left\"\n      end\n\n      def table_opts do\n        [\n          container: true,\n          container_attrs: [class: \"table-container\"],\n          no_results_content: content_tag(:p, do: \"Nothing found.\"),\n          table_attrs: [class: \"table\"]\n        ]\n      end\n    end\n\nRefer to `t:pagination_option/0` and `t:table_option/0` for a list of\navailable options and defaults.\n\nOnce you have defined these functions, you can reference them with a\nmodule/function tuple in `config/config.exs`.\n\n```elixir\nconfig :flop_phoenix,\n  pagination: [opts: {MyApp.CoreComponents, :pagination_opts}],\n  table: [opts: {MyApp.CoreComponents, :table_opts}]\n```","ref":"Flop.Phoenix.html#module-customization"},{"type":"module","title":"Hiding default parameters - Flop.Phoenix","doc":"Default values for page size and ordering are omitted from the query\nparameters. If you pass the `:for` assign, the Flop.Phoenix function will\npick up the default values from the schema module deriving `Flop.Schema`.","ref":"Flop.Phoenix.html#module-hiding-default-parameters"},{"type":"module","title":"Links - Flop.Phoenix","doc":"Links are generated with `Phoenix.Components.link/1`. This will\nlead to ` ` tags with `data-phx-link` and `data-phx-link-state` attributes,\nwhich will be ignored outside of LiveViews and LiveComponents.\n\nWhen used within a LiveView or LiveComponent, you will need to handle the new\nparams in the `c:Phoenix.LiveView.handle_params/3` callback of your LiveView\nmodule.","ref":"Flop.Phoenix.html#module-links"},{"type":"module","title":"Using JS commands - Flop.Phoenix","doc":"You can pass a `Phoenix.LiveView.JS` command as `on_paginate` and `on_sort`\nattributes.\n\nIf used with the `path` attribute, the URL will be patched _and_ the given\nJS command will be executed.\n\nIf used without the `path` attribute, you will need to include a `push`\ncommand to trigger an event when a pagination or sort link is clicked.\n\nYou can set a different target by assigning a `:target`. The value\nwill be used as the `phx-target` attribute.\n\n     \n\n     \n\nYou will need to handle the event in the `c:Phoenix.LiveView.handle_event/3`\nor `c:Phoenix.LiveComponent.handle_event/3` callback of your\nLiveView or LiveComponent module. The event name will be the one you set with\nthe `:event` option.\n\n    def handle_event(\"paginate-pets\", %{\"page\" => page}, socket) do\n      flop = Flop.set_page(socket.assigns.meta.flop, page)\n\n      with {:ok, {pets, meta}} <- Pets.list_pets(flop) do\n        {:noreply, assign(socket, pets: pets, meta: meta)}\n      end\n    end\n\n    def handle_event(\"sort-pets\", %{\"order\" => order}, socket) do\n      flop = Flop.push_order(socket.assigns.meta.flop, order)\n\n      with {:ok, {pets, meta}} <- Pets.list_pets(flop) do\n        {:noreply, assign(socket, pets: pets, meta: meta)}\n      end\n    end","ref":"Flop.Phoenix.html#module-using-js-commands"},{"type":"function","title":"Flop.Phoenix.build_path/3","doc":"Builds a path that includes query parameters for the given `Flop` struct\nusing the referenced Phoenix path helper function.\n\nThe first argument can be either one of:\n\n- an MFA tuple (module, function name as atom, arguments)\n- a 2-tuple (function, arguments)\n- a URL string, usually produced with a verified route (e.g. `~p\"/some/path\"`)\n- a function that takes the Flop parameters as a keyword list as an argument\n\nDefault values for `limit`, `page_size`, `order_by` and `order_directions` are\nomitted from the query parameters. To pick up the default parameters from a\nschema module deriving `Flop.Schema`, you need to pass the `:for` option. To\npick up the default parameters from the backend module, you need to pass the\n`:backend` option. If you pass a `Flop.Meta` struct as the second argument,\nthese options are retrieved from the struct automatically.\n\n> ##","ref":"Flop.Phoenix.html#build_path/3"},{"type":"function","title":"Date and Time Filters {: .info} - Flop.Phoenix.build_path/3","doc":">\n> When using filters on `Date`, `DateTime`, `NaiveDateTime` or `Time` fields,\n> you may need to implement the `Phoenix.Param` protocol for these structs.\n> See the documentation for `to_query/2`.","ref":"Flop.Phoenix.html#build_path/3-date-and-time-filters-info"},{"type":"function","title":"Examples - Flop.Phoenix.build_path/3","doc":"#","ref":"Flop.Phoenix.html#build_path/3-examples"},{"type":"function","title":"With a verified route - Flop.Phoenix.build_path/3","doc":"The examples below use plain URL strings without the p-sigil, so that the\ndoc tests work, but in your application, you can use verified routes or\nanything else that produces a URL.\n\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> path = build_path(\"/pets\", flop)\n    iex> %URI{path: parsed_path, query: parsed_query} = URI.parse(path)\n    iex> {parsed_path, URI.decode_query(parsed_query)}\n    {\"/pets\", %{\"page\" => \"2\", \"page_size\" => \"10\"}}\n\nThe Flop query parameters will be merged into existing query parameters.\n\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> path = build_path(\"/pets?species=dogs\", flop)\n    iex> %URI{path: parsed_path, query: parsed_query} = URI.parse(path)\n    iex> {parsed_path, URI.decode_query(parsed_query)}\n    {\"/pets\", %{\"page\" => \"2\", \"page_size\" => \"10\", \"species\" => \"dogs\"}}\n\n#","ref":"Flop.Phoenix.html#build_path/3-with-a-verified-route"},{"type":"function","title":"With an MFA tuple - Flop.Phoenix.build_path/3","doc":"iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path(\n    ...>   {Flop.PhoenixTest, :route_helper, [%Plug.Conn{}, :pets]},\n    ...>   flop\n    ...> )\n    \"/pets?page_size=10&page=2\"\n\n#","ref":"Flop.Phoenix.html#build_path/3-with-an-mfa-tuple"},{"type":"function","title":"With a function/arguments tuple - Flop.Phoenix.build_path/3","doc":"iex> pet_path = fn _conn, :index, query ->\n    ...>   \"/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path({pet_path, [%Plug.Conn{}, :index]}, flop)\n    \"/pets?page_size=10&page=2\"\n\nWe're defining fake path helpers for the scope of the doctests. In a real\nPhoenix application, you would pass something like\n`{Routes, :pet_path, args}` or `{&Routes.pet_path/3, args}` as the\nfirst argument.\n\n#","ref":"Flop.Phoenix.html#build_path/3-with-a-function-arguments-tuple"},{"type":"function","title":"Passing a `Flop.Meta` struct or a keyword list - Flop.Phoenix.build_path/3","doc":"You can also pass a `Flop.Meta` struct or a keyword list as the third\nargument.\n\n    iex> pet_path = fn _conn, :index, query ->\n    ...>   \"/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> meta = %Flop.Meta{flop: flop}\n    iex> build_path({pet_path, [%Plug.Conn{}, :index]}, meta)\n    \"/pets?page_size=10&page=2\"\n    iex> query_params = to_query(flop)\n    iex> build_path({pet_path, [%Plug.Conn{}, :index]}, query_params)\n    \"/pets?page_size=10&page=2\"\n\n#","ref":"Flop.Phoenix.html#build_path/3-passing-a-flop-meta-struct-or-a-keyword-list"},{"type":"function","title":"Additional path parameters - Flop.Phoenix.build_path/3","doc":"If the path helper takes additional path parameters, just add them to the\nsecond argument.\n\n    iex> user_pet_path = fn _conn, :index, id, query ->\n    ...>   \"/users/#{id}/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path({user_pet_path, [%Plug.Conn{}, :index, 123]}, flop)\n    \"/users/123/pets?page_size=10&page=2\"\n\n#","ref":"Flop.Phoenix.html#build_path/3-additional-path-parameters"},{"type":"function","title":"Additional query parameters - Flop.Phoenix.build_path/3","doc":"If the last path helper argument is a query parameter list, the Flop\nparameters are merged into it.\n\n    iex> pet_url = fn _conn, :index, query ->\n    ...>   \"https://pets.flop/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{order_by: :name, order_directions: [:desc]}\n    iex> build_path({pet_url, [%Plug.Conn{}, :index, [user_id: 123]]}, flop)\n    \"https://pets.flop/pets?user_id=123&order_directions[]=desc&order_by=name\"\n    iex> build_path(\n    ...>   {pet_url,\n    ...>    [%Plug.Conn{}, :index, [category: \"small\", user_id: 123]]},\n    ...>   flop\n    ...> )\n    \"https://pets.flop/pets?category=small&user_id=123&order_directions[]=desc&order_by=name\"\n\n#","ref":"Flop.Phoenix.html#build_path/3-additional-query-parameters"},{"type":"function","title":"Set page as path parameter - Flop.Phoenix.build_path/3","doc":"Finally, you can also pass a function that takes the Flop parameters as\na keyword list as an argument. Default values will not be included in the\nparameters passed to the function. You can use this if you need to set some\nof the parameters as path parameters instead of query parameters.\n\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path(\n    ...>   fn params ->\n    ...>     {page, params} = Keyword.pop(params, :page)\n    ...>     query = Plug.Conn.Query.encode(params)\n    ...>     if page, do: \"/pets/page/#{page}?#{query}\", else: \"/pets?#{query}\"\n    ...>   end,\n    ...>   flop\n    ...> )\n    \"/pets/page/2?page_size=10\"\n\nNote that in this example, the anonymous function just returns a string. With\nPhoenix 1.7, you will be able to use verified routes.\n\n    build_path(\n      fn params ->\n        {page, query} = Keyword.pop(params, :page)\n        if page, do: ~p\"/pets/page/#{page}?#{query}\", else: ~p\"/pets?#{query}\"\n      end,\n      flop\n    )\n\nNote that the keyword list passed to the path builder function is built using\n`Plug.Conn.Query.encode/2`, which means filters are formatted as map with\ninteger keys.\n\n#","ref":"Flop.Phoenix.html#build_path/3-set-page-as-path-parameter"},{"type":"function","title":"Set filter value as path parameter - Flop.Phoenix.build_path/3","doc":"If you need to set a filter value as a path parameter, you can use\n`Flop.Filter.pop/3`.\n\n    iex> flop = %Flop{\n    ...>   page: 5,\n    ...>   order_by: [:published_at],\n    ...>   filters: [\n    ...>     %Flop.Filter{field: :category, op: :==, value: \"announcements\"}\n    ...>   ]\n    ...> }\n    iex> build_path(\n    ...>   fn params ->\n    ...>     {page, params} = Keyword.pop(params, :page)\n    ...>     filters = Keyword.get(params, :filters, [])\n    ...>     {category, filters} = Flop.Filter.pop(filters, :category)\n    ...>     params = Keyword.put(params, :filters, filters)\n    ...>     query = Plug.Conn.Query.encode(params)\n    ...>\n    ...>     case {page, category} do\n    ...>       {nil, nil} -> \"/articles?#{query}\"\n    ...>       {page, nil} -> \"/articles/page/#{page}?#{query}\"\n    ...>       {nil, %{value: category}} -> \"/articles/category/#{category}?#{query}\"\n    ...>       {page, %{value: category}} -> \"/articles/category/#{category}/page/#{page}?#{query}\"\n    ...>     end\n    ...>   end,\n    ...>   flop\n    ...> )\n    \"/articles/category/announcements/page/5?order_by[]=published_at\"","ref":"Flop.Phoenix.html#build_path/3-set-filter-value-as-path-parameter"},{"type":"function","title":"Flop.Phoenix.cursor_pagination/1","doc":"Renders a cursor pagination element.","ref":"Flop.Phoenix.html#cursor_pagination/1"},{"type":"function","title":"Examples - Flop.Phoenix.cursor_pagination/1","doc":"","ref":"Flop.Phoenix.html#cursor_pagination/1-examples"},{"type":"function","title":"Handling parameters and JS commands - Flop.Phoenix.cursor_pagination/1","doc":"If you set the `path` assign, a link with query parameters is rendered.\nIn a LiveView, you need to handle the parameters in the\n`c:Phoenix.LiveView.handle_params/3` callback.\n\n    def handle_params(params, _, socket) do\n      {pets, meta} = MyApp.list_pets(params)\n      {:noreply, assign(socket, meta: meta, pets: pets)}\n    end\n\nIf you use LiveView and set the `on_paginate` attribute, you need to update\nthe Flop parameters in the `handle_event/3` callback.\n\n    def handle_event(\"paginate-users\", %{\"to\" => to}, socket) do\n      flop = Flop.set_cursor(socket.assigns.meta, to)\n      {pets, meta} = MyApp.list_pets(flop)\n      {:noreply, assign(socket, meta: meta, pets: pets)}\n    end","ref":"Flop.Phoenix.html#cursor_pagination/1-handling-parameters-and-js-commands"},{"type":"function","title":"Getting the right parameters from Flop - Flop.Phoenix.cursor_pagination/1","doc":"This component requires the start and end cursors to be set in `Flop.Meta`. If\nyou pass a `Flop.Meta` struct with page or offset-based parameters, this will\nresult in an error. You can enforce cursor-based pagination in your query\nfunction with the `default_pagination_type` and `pagination_types` options.\n\n    def list_pets(params) do\n      Flop.validate_and_run!(Pet, params,\n        for: Pet,\n        default_pagination_type: :first,\n        pagination_types: [:first, :last]\n      )\n    end\n\n`default_pagination_type` ensures that Flop defaults to the right pagination\ntype when it cannot determine the type from the parameters. `pagination_types`\nensures that parameters for other types are not accepted.","ref":"Flop.Phoenix.html#cursor_pagination/1-getting-the-right-parameters-from-flop"},{"type":"function","title":"Order fields - Flop.Phoenix.cursor_pagination/1","doc":"The pagination cursor is based on the `ORDER BY` fields of the query. It is\nimportant that the combination of order fields is unique across the data set.\nYou can use:\n\n- the field with the primary key\n- a field with a unique index\n- all fields of a composite primary key or unique index\n\nIf you want to order by fields that are not unique, you can add the primary\nkey as the last order field. For example, if you want to order by family name\nand given name, you should set the `order_by` parameter to\n`[:family_name, :given_name, :id]`.","ref":"Flop.Phoenix.html#cursor_pagination/1-order-fields"},{"type":"function","title":"Attributes - Flop.Phoenix.cursor_pagination/1","doc":"* `meta` (`Flop.Meta`) (required) - The meta information of the query as returned by the `Flop` query functions.\n* `path` (`:any`) - If set, the current view is patched with updated query parameters when a\n  pagination link is clicked. In case the `on_paginate` attribute is set as\n  well, the URL is patched _and_ the given JS command is executed.\n\n  The value must be either a URI string (Phoenix verified route), an MFA or FA\n  tuple (Phoenix route helper), or a 1-ary path builder function. See\n  `Flop.Phoenix.build_path/3` for details.\n\n  Defaults to `nil`.\n* `on_paginate` (`Phoenix.LiveView.JS`) - A `Phoenix.LiveView.JS` command that is triggered when a pagination link is\n  clicked.\n\n  If used without the `path` attribute, you should include a `push` operation\n  to handle the event with the `handle_event` callback.\n\n      <.cursor_pagination\n        meta={@meta}\n        on_paginate={\n          JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\") |> JS.push(\"paginate\")\n        }\n      />\n\n  If used with the `path` attribute, the URL is patched _and_ the given JS\n  command is executed.\n\n      <.cursor_pagination\n        meta={@meta}\n        path={~\"/pets\"}\n        on_paginate={JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\")}\n      />\n\n  With the above attributes in place, you can add the following JavaScript to\n  your application to scroll to the top of your table whenever a pagination\n  link is clicked:\n\n  ```js\n  window.addEventListener(\"my_app:scroll_to\", (e) => {\n    e.target.scrollIntoView();\n  });\n  ```\n\n  You can use CSS to scroll to the new position smoothly.\n\n  ```css\n  html {\n    scroll-behavior: smooth;\n  }\n  ```\n\n  Defaults to `nil`.\n* `event` (`:string`) - If set, `Flop.Phoenix` will render links with a `phx-click` attribute.\n  Alternatively, set `:path`, if you want the parameters to appear in the URL.\n  Deprecated. Use `on_paginate` instead.\n\n  Defaults to `nil`.\n* `target` (`:string`) - Sets the `phx-target` attribute for the pagination links. Defaults to `nil`.\n* `reverse` (`:boolean`) - By default, the `next` link moves forward with the `:after` parameter set to\n  the end cursor, and the `previous` link moves backward with the `:before`\n  parameter set to the start cursor. If `reverse` is set to `true`, the\n  destinations of the links are switched.\n\n  Defaults to `false`.\n* `opts` (`:list`) - Options to customize the pagination. See\n  `t:Flop.Phoenix.cursor_pagination_option/0`. Note that the options passed to\n  the function are deep merged into the default options. Since these options\n  will likely be the same for all the cursor pagination links in a project,\n  it is recommended to define them once in a function or set them in a\n  wrapper function as described in the `Customization` section of the module\n  documentation.\n\n  Defaults to `[]`.","ref":"Flop.Phoenix.html#cursor_pagination/1-attributes"},{"type":"function","title":"Flop.Phoenix.filter_fields/1","doc":"Renders all inputs for a filter form including the hidden inputs.","ref":"Flop.Phoenix.html#filter_fields/1"},{"type":"function","title":"Example - Flop.Phoenix.filter_fields/1","doc":"def filter_form(%{meta: meta} = assigns) do\n      assigns = assign(assigns, :form, Phoenix.Component.to_form(meta))\n\n      ~H\"\"\"\n      <.form for={@form}>\n        <.filter_fields :let={i} form={@form} fields={[:email, :name]}>\n          <.input\n            field={i.field}\n            label={i.label}\n            type={i.type}\n            {i.rest}\n          />\n        </.filter_fields>\n      </.form>\n      \"\"\"\n    end\n\nThis assumes that you have defined an `input` component that renders a form\ninput including the label.\n\nThese options are passed to the inner block via `:let`:\n\n- The `field` is a `Phoenix.HTML.FormField.t` struct.\n- The `type` is the input type as a string, _not_ the name of the\n  `Phoenix.HTML.Form` input function (e.g. `\"text\"`, not `:text_input`). The\n  type is derived from the type of the field being filtered on, but it can\n  be overridden in the field options.\n- `rest` contains any additional field options passed.","ref":"Flop.Phoenix.html#filter_fields/1-example"},{"type":"function","title":"Field configuration - Flop.Phoenix.filter_fields/1","doc":"The fields can be passed as atoms or keywords with additional options.\n\n    fields={[:name, :email]}\n\nOr\n\n    fields={[\n      name: [label: gettext(\"Name\")],\n      email: [\n        label: gettext(\"Email\"),\n        op: :ilike_and,\n        type: \"email\"\n      ],\n      age: [\n        label: gettext(\"Age\"),\n        type: \"select\",\n        prompt: \"\",\n        options: [\n          {gettext(\"young\"), :young},\n          {gettext(\"old\"), :old)}\n        ]\n      ]\n    ]}\n\nAvailable options:\n\n- `label` - Defaults to the humanized field name.\n- `op` - Defaults to `:==`.\n- `type` - Defaults to an input type depending on the Ecto type of the filter\n  field.\n\nAny additional options will be passed to the input component (e.g. HTML\nclasses or a list of options).","ref":"Flop.Phoenix.html#filter_fields/1-field-configuration"},{"type":"function","title":"Attributes - Flop.Phoenix.filter_fields/1","doc":"* `form` (`Phoenix.HTML.Form`) (required)\n* `fields` (`:list`) - The list of fields and field options. Note that inputs will not be rendered\n  for fields that are not marked as filterable in the schema\n  (see `Flop.Schema`).\n\n  If `dynamic` is set to `false`, only fields in this list are rendered. If\n  `dynamic` is set to `true`, only fields for filters present in the given\n  `Flop.Meta` struct are rendered, and the fields are rendered even if they\n  are not passed in the `fields` list. In the latter case, `fields` is\n  optional, but you can still pass label and input configuration this way.\n\n  Note that in a dynamic form, it is not possible to configure a single field\n  multiple times.\n\n  Defaults to `[]`.\n* `dynamic` (`:boolean`) - If `true`, fields are only rendered for filters that are present in the\n  `Flop.Meta` struct passed to the form. You can use this for rendering filter\n  forms that allow the user to add and remove filters dynamically. The\n  `fields` assign is only used for looking up the options in that case.\n\n  Defaults to `false`.","ref":"Flop.Phoenix.html#filter_fields/1-attributes"},{"type":"function","title":"Slots - Flop.Phoenix.filter_fields/1","doc":"* `inner_block` - The necessary options for rendering a label and an input are passed to the\n  inner block, which allows you to render the fields with your existing\n  components.\n\n      <.filter_fields :let={i} form={@form} fields={[:email, :name]}>\n        <.input\n          field={i.field}\n          label={i.label}\n          type={i.type}\n          {i.rest}\n        />\n      </.filter_fields>\n\n  The options passed to the inner block are:\n\n  - `field` - A `Phoenix.HTML.FormField` struct.\n  - `type` - The input type as a string. This is _not_ the value returned\n    by `Phoenix.HTML.Form.input_type/2`.\n  - `label` - The label text as a string.\n  - `rest` - Any additional options passed in the field options.","ref":"Flop.Phoenix.html#filter_fields/1-slots"},{"type":"function","title":"Flop.Phoenix.hidden_inputs_for_filter/1","doc":"Renders hidden inputs for the given form.\n\nYou can use this for convenience if you have a complex form layout that cannot\nbe accomplished with `Flop.Phoenix.filter_fields/1`. Put it as a direct child\nof the `form` component to render the hidden inputs for pagination and order\nparameters. Then use `Phoenix.HTML.Form.inputs_for/3` to render a single\nfilter field, and place this component within the `do` block to render the\nhidden inputs for the filter field and operator.\n\nSince the filters are represented as an array in the params, make sure to\nadd the `offset` option so that the `Flop.Meta` can be properly mapped back to\nyour input fields. For every call to `inputs_for` always add the length of all\nprevious calls to `inputs_for` as offset.\n\n    <.form :let={f} for={@meta}>\n      <.hidden_inputs_for_filter form={@form} />\n\n       \n         \n          <%= for ff <- Phoenix.HTML.Form.inputs_for(f, :filters, fields: [:name]) do %>\n            <.hidden_inputs_for_filter form={ff} />\n            <.input label=\"Name\" type=\"text\" field={{ff, :value}} />\n          <% end %>\n         \n         \n          <%= for ff <- Phoenix.HTML.Form.inputs_for(f, :filters, fields: [{:email, op: :ilike}], offset: 1) do %>\n            <.hidden_inputs_for_filter form={ff} />\n            <.input label=\"E-mail\" type=\"email\" field={{ff, :value}} />\n          <% end %>\n         \n       \n    </.form>","ref":"Flop.Phoenix.html#hidden_inputs_for_filter/1"},{"type":"function","title":"Attributes - Flop.Phoenix.hidden_inputs_for_filter/1","doc":"* `form` (`Phoenix.HTML.Form`) (required)","ref":"Flop.Phoenix.html#hidden_inputs_for_filter/1-attributes"},{"type":"function","title":"Flop.Phoenix.pagination/1","doc":"Generates a pagination element.","ref":"Flop.Phoenix.html#pagination/1"},{"type":"function","title":"Examples - Flop.Phoenix.pagination/1","doc":"","ref":"Flop.Phoenix.html#pagination/1-examples"},{"type":"function","title":"Page link options - Flop.Phoenix.pagination/1","doc":"By default, page links for all pages are shown. You can limit the number of\npage links or disable them altogether by passing the `:page_links` option.\n\n- `:all`: Show all page links (default).\n- `:hide`: Don't show any page links. Only the previous/next links will be\n  shown.\n- `{:ellipsis, x}`: Limits the number of page links. The first and last page\n  are always displayed. The `x` refers to the number of additional page links\n  to show.","ref":"Flop.Phoenix.html#pagination/1-page-link-options"},{"type":"function","title":"Pagination link aria label - Flop.Phoenix.pagination/1","doc":"For the page links, there is the `:pagination_link_aria_label` option to set\nthe aria label. Since the page number is usually part of the aria label, you\nneed to pass a function that takes the page number as an integer and returns\nthe label as a string. The default is `&\"Goto page #{&1}\"`.","ref":"Flop.Phoenix.html#pagination/1-pagination-link-aria-label"},{"type":"function","title":"Previous/next links - Flop.Phoenix.pagination/1","doc":"By default, the previous and next links contain the texts `Previous` and\n`Next`. To change this, you can pass the `:previous_link_content` and\n`:next_link_content` options.","ref":"Flop.Phoenix.html#pagination/1-previous-next-links"},{"type":"function","title":"Attributes - Flop.Phoenix.pagination/1","doc":"* `meta` (`Flop.Meta`) (required) - The meta information of the query as returned by the `Flop` query functions.\n* `path` (`:any`) - If set, the current view is patched with updated query parameters when a\n  pagination link is clicked. In case the `on_paginate` attribute is set as\n  well, the URL is patched _and_ the given command is executed.\n\n  The value must be either a URI string (Phoenix verified route), an MFA or FA\n  tuple (Phoenix route helper), or a 1-ary path builder function. See\n  `Flop.Phoenix.build_path/3` for details.\n\n  Defaults to `nil`.\n* `on_paginate` (`Phoenix.LiveView.JS`) - A `Phoenix.LiveView.JS` command that is triggered when a pagination link is\n  clicked.\n\n  If used without the `path` attribute, you should include a `push` operation\n  to handle the event with the `handle_event` callback.\n\n      <.pagination\n        meta={@meta}\n        on_paginate={\n          JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\") |> JS.push(\"paginate\")\n        }\n      />\n\n  If used with the `path` attribute, the URL is patched _and_ the given\n  JS command is executed.\n\n      <.pagination\n        meta={@meta}\n        path={~\"/pets\"}\n        on_paginate={JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\")}\n      />\n\n  With the above attributes in place, you can add the following JavaScript to\n  your application to scroll to the top of your table whenever a pagination\n  link is clicked:\n\n  ```js\n  window.addEventListener(\"my_app:scroll_to\", (e) => {\n    e.target.scrollIntoView();\n  });\n  ```\n\n  You can use CSS to scroll to the new position smoothly.\n\n  ```css\n  html {\n    scroll-behavior: smooth;\n  }\n  ```\n\n  Defaults to `nil`.\n* `event` (`:string`) - If set, `Flop.Phoenix` will render links with a `phx-click` attribute.\n  Alternatively, set `:path`, if you want the parameters to appear in the URL.\n  Deprecated in favor of `on_paginate`.\n\n  Defaults to `nil`.\n* `target` (`:string`) - Sets the `phx-target` attribute for the pagination links. Defaults to `nil`.\n* `opts` (`:list`) - Options to customize the pagination. See\n  `t:Flop.Phoenix.pagination_option/0`. Note that the options passed to the\n  function are deep merged into the default options. Since these options will\n  likely be the same for all the tables in a project, it is recommended to\n  define them once in a function or set them in a wrapper function as\n  described in the `Customization` section of the module documentation.\n\n  Defaults to `[]`.","ref":"Flop.Phoenix.html#pagination/1-attributes"},{"type":"function","title":"Flop.Phoenix.table/1","doc":"Generates a table with sortable columns.","ref":"Flop.Phoenix.html#table/1"},{"type":"function","title":"Example - Flop.Phoenix.table/1","doc":"```elixir\n \n  <:col :let={pet} label=\"Name\" field={:name}><%= pet.name %></:col>\n  <:col :let={pet} label=\"Age\" field={:age}><%= pet.age %></:col>\n \n```","ref":"Flop.Phoenix.html#table/1-example"},{"type":"function","title":"Flop.Schema - Flop.Phoenix.table/1","doc":"If you pass the `for` option when making the query with Flop, Flop Phoenix can\ndetermine which table columns are sortable. It also hides the `order` and\n`page_size` parameters if they match the default values defined with\n`Flop.Schema`.","ref":"Flop.Phoenix.html#table/1-flop-schema"},{"type":"function","title":"Attributes - Flop.Phoenix.table/1","doc":"* `id` (`:string`) - ID used on the table. If not set, an ID is chosen based on the schema\n  module derived from the `Flop.Meta` struct.\n\n  The ID is necessary in case the table is fed with a LiveView stream.\n\n* `items` (`:list`) (required) - The list of items to be displayed in rows. This is the result list returned\n  by the query.\n\n* `meta` (`Flop.Meta`) (required) - The `Flop.Meta` struct returned by the query function.\n* `path` (`:any`) - If set, the current view is patched with updated query parameters when a\n  header link for sorting is clicked. In case the `on_sort` attribute is\n  set as well, the URL is patched _and_ the given JS command is executed.\n\n  The value must be either a URI string (Phoenix verified route), an MFA or FA\n  tuple (Phoenix route helper), or a 1-ary path builder function. See\n  `Flop.Phoenix.build_path/3` for details.\n\n  Defaults to `nil`.\n* `on_sort` (`Phoenix.LiveView.JS`) - A `Phoenix.LiveView.JS` command that is triggered when a header link for\n  sorting is clicked.\n\n  If used without the `path` attribute, you should include a `push` operation\n  to handle the event with the `handle_event` callback.\n\n      <.table\n        items={@items}\n        meta={@meta}\n        on_sort={\n          JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\") |> JS.push(\"sort\")\n        }\n      />\n\n  If used with the `path` attribute, the URL is patched _and_ the given\n  JS command is executed.\n\n      <.table\n        meta={@meta}\n        path={~\"/pets\"}\n        on_sort={JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\")}\n      />\n\n  Defaults to `nil`.\n* `event` (`:string`) - If set, `Flop.Phoenix` will render links with a `phx-click` attribute.\n  Alternatively, set `:path`, if you want the parameters to appear in the URL.\n  Deprecated in favor of `on_sort`.\n\n  Defaults to `nil`.\n* `target` (`:string`) - Sets the `phx-target` attribute for the header links. Defaults to `nil`.\n* `caption` (`:string`) - Content for the ` ` element. Defaults to `nil`.\n* `opts` (`:list`) - Keyword list with additional options (see `t:Flop.Phoenix.table_option/0`).\n  Note that the options passed to the function are deep merged into the\n  default options. Since these options will likely be the same for all the\n  tables in a project, it is recommended to define them once in a function or\n  set them in a wrapper function as described in the `Customization` section\n  of the module documentation.\n\n  Defaults to `[]`.\n* `row_id` (`:any`) - Overrides the default function that retrieves the row ID from a stream item. Defaults to `nil`.\n* `row_click` (`Phoenix.LiveView.JS`) - Sets the `phx-click` function attribute for each row `td`. Expects to be a\n  function that receives a row item as an argument. This does not add the\n  `phx-click` attribute to the `action` slot.\n\n  Example:\n\n  ```elixir\n  row_click={&JS.navigate(~p\"/users/#{&1}\")}\n  ```\n\n  Defaults to `nil`.\n* `row_item` (`:any`) - This function is called on the row item before it is passed to the :col\n  and :action slots.\n\n  Defaults to `&Function.identity/1`.","ref":"Flop.Phoenix.html#table/1-attributes"},{"type":"function","title":"Slots - Flop.Phoenix.table/1","doc":"* `col` (required) - For each column to render, add one `<:col>` element.\n\n  ```elixir\n  <:col :let={pet} label=\"Name\" field={:name} col_style=\"width: 20%;\">\n    <%= pet.name %>\n  </:col>\n  ```\n\n  Any additional assigns will be added as attributes to the ` ` elements.\n\n  Accepts attributes:\n\n  * `label` (`:any`) - The content for the header column.\n  * `field` (`:atom`) - The field name for sorting. If set and the field is configured as sortable\n    in the schema, the column header will be clickable, allowing the user to\n    sort by that column. If the field is not marked as sortable or if the\n    `field` attribute is omitted or set to `nil` or `false`, the column header\n    will not be clickable.\n\n  * `directions` (`:any`) - An optional 2-element tuple used for custom ascending and descending sort\n    behavior for the column, i.e. `{:asc_nulls_last, :desc_nulls_first}`\n\n  * `show` (`:boolean`) - Boolean value to conditionally show the column. Defaults to `true`\n    Deprecated. Use `:if` instead.\n\n  * `hide` (`:boolean`) - Boolean value to conditionally hide the column. Defaults to `false`.\n    Deprecated. Use `:if` instead.\n\n  * `col_style` (`:string`) - If set, a ` ` element is rendered and the value of the\n    `col_style` assign is set as `style` attribute for the ` ` element of\n    the respective column. You can set the `width`, `background` and `border`\n    of a column this way.\n\n  * `thead_th_attrs` (`:list`) - Additional attributes to pass to the ` ` element as a static keyword\n    list. Note that these attributes will override any conflicting\n    `thead_th_attrs` that are set at the table level.\n\n  * `th_wrapper_attrs` (`:list`) - Additional attributes for the ` ` element that wraps the\n    header link and the order direction symbol. Note that these attributes\n    will override any conflicting `th_wrapper_attrs` that are set at the table\n    level.\n\n  * `tbody_td_attrs` (`:any`) - Additional attributes to pass to the ` ` element. May be provided as a\n    static keyword list, or as a 1-arity function to dynamically generate the\n    list using row data. Note that these attributes will override any\n    conflicting `tbody_td_attrs` that are set at the table level.\n\n* `action` - The slot for showing user actions in the last table column. These columns\n  do not receive the `row_click` attribute.\n\n\n  ```elixir\n  <:action :let={user}>\n    <.link navigate={~p\"/users/#{user}\"}>Show</.link>\n  </:action>\n  ```\n\n  Accepts attributes:\n\n  * `label` (`:string`) - The content for the header column.\n  * `show` (`:boolean`) - Boolean value to conditionally show the column. Defaults to `true`.\n  * `hide` (`:boolean`) - Boolean value to conditionally hide the column. Defaults to `false`.\n  * `col_style` (`:string`) - If set, a ` ` element is rendered and the value of the\n    `col_style` assign is set as `style` attribute for the ` ` element of\n    the respective column. You can set the `width`, `background` and `border`\n    of a column this way.\n\n  * `thead_th_attrs` (`:list`) - Any additional attributes to pass to the ` ` as a keyword list.\n  * `tbody_td_attrs` (`:any`) - Any additional attributes to pass to the ` `. Can be a keyword list or\n    a function that takes the current row item as an argument and returns a\n    keyword list.\n\n* `foot` - You can optionally add a `foot`. The inner block will be rendered inside\n  a `tfoot` element.\n\n       \n        <:foot>\n            Total:  <%= @total %>   \n        </:foot>","ref":"Flop.Phoenix.html#table/1-slots"},{"type":"function","title":"Flop.Phoenix.to_query/2","doc":"Converts a Flop struct into a keyword list that can be used as a query with\nPhoenix verified routes or route helper functions.","ref":"Flop.Phoenix.html#to_query/2"},{"type":"function","title":"Default parameters - Flop.Phoenix.to_query/2","doc":"Default parameters for the limit and order parameters are omitted. The\ndefaults are determined by calling `Flop.get_option/3`.\n\n- Pass the `:for` option to pick up the default values from a schema module\n  deriving `Flop.Schema`.\n- Pass the `:backend` option to pick up the default values from your backend\n  configuration.\n- If neither the schema module nor the backend module have default options\n  set, the function will fall back to the application environment.","ref":"Flop.Phoenix.html#to_query/2-default-parameters"},{"type":"function","title":"Encoding queries - Flop.Phoenix.to_query/2","doc":"To encode the returned query as a string, you will need to use\n`Plug.Conn.Query.encode/1`. `URI.encode_query/1` does not support bracket\nnotation for arrays and maps.","ref":"Flop.Phoenix.html#to_query/2-encoding-queries"},{"type":"function","title":"Date and time filters - Flop.Phoenix.to_query/2","doc":"If you use the result of this function directly with\n`Phoenix.VerifiedRoutes.sigil_p/2` for verified routes or in a route helper\nfunction, all cast filter values need to be able to be converted to a string\nusing the `Phoenix.Param` protocol.\n\nThis protocol is implemented by default for integers, binaries, atoms, and\nstructs. For structs, Phoenix's default behavior is to fetch the id field.\n\nIf you have filters with `Date`, `DateTime`, `NaiveDateTime`,\n`Time` values, or any other custom structs (e.g. structs that represent\ncomposite types like a range column), you will need to implement the protocol\nfor these specific structs in your application.\n\n    defimpl Phoenix.Param, for: Date do\n      def to_param(%Date{} = d), do: to_string(d)\n    end\n\n    defimpl Phoenix.Param, for: DateTime do\n      def to_param(%DateTime{} = dt), do: to_string(dt)\n    end\n\n    defimpl Phoenix.Param, for: NaiveDateTime do\n      def to_param(%NaiveDateTime{} = dt), do: to_string(dt)\n    end\n\n    defimpl Phoenix.Param, for: Time do\n      def to_param(%Time{} = t), do: to_string(t)\n    end\n\nIt is important that the chosen string representation can be cast back into\nthe Ecto type.","ref":"Flop.Phoenix.html#to_query/2-date-and-time-filters"},{"type":"function","title":"Examples - Flop.Phoenix.to_query/2","doc":"iex> to_query(%Flop{})\n    []\n\n    iex> f = %Flop{page: 5, page_size: 20}\n    iex> to_query(f)\n    [page_size: 20, page: 5]\n\n    iex> f = %Flop{first: 20, after: \"g3QAAAABZAAEbmFtZW0AAAAFQXBwbGU=\"}\n    iex> to_query(f)\n    [first: 20, after: \"g3QAAAABZAAEbmFtZW0AAAAFQXBwbGU=\"]\n\n    iex> f = %Flop{\n    ...>   filters: [\n    ...>     %Flop.Filter{field: :name, op: :=~, value: \"Mag\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 25}\n    ...>   ]\n    ...> }\n    iex> to_query(f)\n    [\n      filters: %{\n        0 => %{field: :name, op: :=~, value: \"Mag\"},\n        1 => %{field: :age, op: :>, value: 25}\n      }\n    ]\n    iex> to_query(f)\n    [filters: %{0 => %{value: \"Mag\", op: :=~, field: :name}, 1 => %{value: 25, op: :>, field: :age}}]\n\n    iex> f = %Flop{page: 5, page_size: 20}\n    iex> to_query(f, default_limit: 20)\n    [page: 5]\n\nEncoding the query as a string:\n\n    iex> f = %Flop{order_by: [:name, :age], order_directions: [:desc, :asc]}\n    iex> to_query(f)\n    [order_directions: [:desc, :asc], order_by: [:name, :age]]\n    iex> f |> to_query |> Plug.Conn.Query.encode()\n    \"order_directions[]=desc&order_directions[]=asc&order_by[]=name&order_by[]=age\"","ref":"Flop.Phoenix.html#to_query/2-examples"},{"type":"type","title":"Flop.Phoenix.cursor_pagination_option/0","doc":"Defines the available options for `Flop.Phoenix.cursor_pagination/1`.\n\n- `:disabled` - The class which is added to disabled links. Default:\n  `\"disabled\"`.\n- `:next_link_attrs` - The attributes for the link to the next page.\n  Default: `[aria: [label: \"Go to next page\"], class: \"pagination-next\"]`.\n- `:next_link_content` - The content for the link to the next page.\n  Default: `\"Next\"`.\n- `:previous_link_attrs` - The attributes for the link to the previous page.\n  Default: `[aria: [label: \"Go to previous page\"], class: \"pagination-previous\"]`.\n- `:previous_link_content` - The content for the link to the previous page.\n  Default: `\"Previous\"`.\n- `:wrapper_attrs` - The attributes for the ` ` element that wraps the\n  pagination links.\n  Default: `[class: \"pagination\", role: \"navigation\", aria: [label: \"pagination\"]]`.","ref":"Flop.Phoenix.html#t:cursor_pagination_option/0"},{"type":"type","title":"Flop.Phoenix.pagination_option/0","doc":"Defines the available options for `Flop.Phoenix.pagination/1`.\n\n- `:current_link_attrs` - The attributes for the link to the current page.\n  Default: `[class: \"pagination-link is-current\", aria: [current: \"page\"]]`.\n- `:disabled` - The class which is added to disabled links. Default:\n  `\"disabled\"`.\n- `:ellipsis_attrs` - The attributes for the ` ` that wraps the\n  ellipsis.\n  Default: `[class: \"pagination-ellipsis\"]`.\n- `:ellipsis_content` - The content for the ellipsis element.\n  Default: `{:safe, \"&hellip;\"}`.\n- `:next_link_attrs` - The attributes for the link to the next page.\n  Default: `[aria: [label: \"Go to next page\"], class: \"pagination-next\"]`.\n- `:next_link_content` - The content for the link to the next page.\n  Default: `\"Next\"`.\n- `:page_links` - Specifies how many page links should be rendered.\n  Default: `:all`.\n  - `:all` - Renders all page links.\n  - `{:ellipsis, n}` - Renders `n` page links. Renders ellipsis elements if\n    there are more pages than displayed.\n  - `:hide` - Does not render any page links.\n- `:pagination_link_aria_label` - 1-arity function that takes a page number\n  and returns an aria label for the corresponding page link.\n  Default: `&\"Go to page #{&1}\"`.\n- `:pagination_link_attrs` - The attributes for the pagination links.\n  Default: `[class: \"pagination-link\"]`.\n- `:pagination_list_attrs` - The attributes for the pagination list.\n  Default: `[class: \"pagination-list\"]`.\n- `:previous_link_attrs` - The attributes for the link to the previous page.\n  Default: `[aria: [label: \"Go to previous page\"], class: \"pagination-previous\"]`.\n- `:previous_link_content` - The content for the link to the previous page.\n  Default: `\"Previous\"`.\n- `:wrapper_attrs` - The attributes for the ` ` element that wraps the\n  pagination links.\n  Default: `[class: \"pagination\", role: \"navigation\", aria: [label: \"pagination\"]]`.","ref":"Flop.Phoenix.html#t:pagination_option/0"},{"type":"type","title":"Flop.Phoenix.table_option/0","doc":"Defines the available options for `Flop.Phoenix.table/1`.\n\n- `:container` - Wraps the table in a ` ` if `true`.\n  Default: `false`.\n- `:container_attrs` - The attributes for the table container.\n  Default: `[class: \"table-container\"]`.\n- `:no_results_content` - Any content that should be rendered if there are no\n  results. Default: ` No results. `.\n- `:table_attrs` - The attributes for the ` ` element.\n  Default: `[]`.\n- `:th_wrapper_attrs` - The attributes for the ` ` element that wraps the\n  header link and the order direction symbol.\n  Default: `[]`.\n- `:symbol_asc` - The symbol that is used to indicate that the column is\n  sorted in ascending order.\n  Default: `\"▴\"`.\n- `:symbol_attrs` - The attributes for the ` ` element that wraps the\n  order direction indicator in the header columns.\n  Default: `[class: \"order-direction\"]`.\n- `:symbol_desc` - The symbol that is used to indicate that the column is\n  sorted in ascending order.\n  Default: `\"▾\"`.\n- `:symbol_unsorted` - The symbol that is used to indicate that the column is\n  not sorted. Default: `nil`.\n- `:tbody_attrs`: Attributes to be added to the ` ` tag within the\n  ` `. Default: `[]`.\n- `:tbody_td_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. Default: `[]`.\n- `:thead_attrs`: Attributes to be added to the ` ` tag within the\n  ` `. Default: `[]`.\n- `:tbody_tr_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. A function with arity of 1 may be passed to dynamically generate\n  the attrs based on row data.\n  Default: `[]`.\n- `:thead_th_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. Default: `[]`.\n- `:thead_tr_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. Default: `[]`.","ref":"Flop.Phoenix.html#t:table_option/0"},{"type":"exception","title":"Flop.Phoenix.InvalidFilterFieldConfigError","doc":"Raised when the `fields` attribute of the `Flop.Phoenix.filter_fields`\ncomponent is set to an invalid value.","ref":"Flop.Phoenix.InvalidFilterFieldConfigError.html"},{"type":"function","title":"Flop.Phoenix.InvalidFilterFieldConfigError.message/1","doc":"","ref":"Flop.Phoenix.InvalidFilterFieldConfigError.html#message/1"},{"type":"exception","title":"Flop.Phoenix.NoMetaFormError","doc":"Raised when a `Phoenix.HTML.Form` struct is passed to\n`Flop.Phoenix.filter_fields/1` that was not built with `Flop.Meta` struct.","ref":"Flop.Phoenix.NoMetaFormError.html"},{"type":"function","title":"Flop.Phoenix.NoMetaFormError.message/1","doc":"","ref":"Flop.Phoenix.NoMetaFormError.html#message/1"},{"type":"exception","title":"Flop.Phoenix.PathOrJSError","doc":"Raised when a neither the `path` nor the `on_*` attribute is set for a\npagination or table component.","ref":"Flop.Phoenix.PathOrJSError.html"},{"type":"function","title":"Flop.Phoenix.PathOrJSError.message/1","doc":"","ref":"Flop.Phoenix.PathOrJSError.html#message/1"},{"type":"extras","title":"Flop Phoenix","doc":"# Flop Phoenix\n\n![CI](https://github.com/woylie/flop_phoenix/workflows/CI/badge.svg) [![Hex](https://img.shields.io/hexpm/v/flop_phoenix)](https://hex.pm/packages/flop_phoenix) [![Coverage Status](https://coveralls.io/repos/github/woylie/flop_phoenix/badge.svg)](https://coveralls.io/github/woylie/flop_phoenix)\n\nFlop Phoenix provides Phoenix components for pagination, sortable tables, and\nfilter forms with [Flop](https://hex.pm/packages/flop) and\n[Ecto](https://hex.pm/packages/ecto).","ref":"readme.html"},{"type":"extras","title":"Installation - Flop Phoenix","doc":"To start using flop_phoenix, add it to your list of dependencies in `mix.exs`\nof your Phoenix application:\n\nAdd `flop_phoenix` to your list of dependencies in the `mix.exs` of your Phoenix\napplication.\n\n```elixir\ndef deps do\n  [\n    {:flop_phoenix, \"~> 0.22.5\"}\n  ]\nend\n```\n\nNext, set up your business logic according to the\n[Flop documentation](https://hex.pm/packages/flop).","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Flop Phoenix","doc":"First, define a function that utilizes `Flop.validate_and_run/3` to query your\ndesired list. For example:\n\n```elixir\ndefmodule MyApp.Pets do\n  alias MyApp.Pet\n\n  def list_pets(params) do\n    Flop.validate_and_run(Pet, params, for: Pet)\n  end\nend\n```\n\n#","ref":"readme.html#usage"},{"type":"extras","title":"LiveView - Flop Phoenix","doc":"In the LiveView, fetch the data and assign it alongside the meta data to the\nsocket.\n\n```elixir\ndefmodule MyAppWeb.PetLive.Index do\n  use MyAppWeb, :live_view\n\n  alias MyApp.Pets\n\n  @impl Phoenix.LiveView\n  def handle_params(params, _, socket) do\n    case Pets.list_pets(params) do\n      {:ok, {pets, meta}} ->\n        {:noreply, assign(socket, %{pets: pets, meta: meta})}\n\n      {:error, _meta} ->\n        # This will reset invalid parameters. Alternatively, you can assign\n        # only the meta and render the errors, or you can ignore the error\n        # case entirely.\n        {:noreply, push_navigate(socket, to: ~p\"/pets\")}\n    end\n  end\nend\n```\n\nIf you prefer the `Flop.Phoenix` components not to reflect pagination, sorting,\nand filtering parameters in the URL, fetch and assign the data in the\n`c:Phoenix.LiveView.handle_event/3` callback. You need to pass a\n`Phoenix.LiveView.JS` command as an attribute to the components in that case.\n\n#","ref":"readme.html#liveview"},{"type":"extras","title":"Controller - Flop Phoenix","doc":"For non-LiveView (\"dead\") views, pass the data and Flop meta struct to your\ntemplate in the controller.\n\n```elixir\ndefmodule MyAppWeb.PetController do\n  use MyAppWeb, :controller\n\n  alias MyApp.Pets\n\n  action_fallback MyAppWeb.FallbackController\n\n  def index(conn, params) do\n    with {:ok, {pets, meta}} <- Pets.list_pets(params) do\n      render(conn, :index, meta: meta, pets: pets)\n    end\n  end\nend\n```","ref":"readme.html#controller"},{"type":"extras","title":"Sortable tables and pagination - Flop Phoenix","doc":"To add a sortable table and pagination links, you can add the following to your\ntemplate:\n\n```elixir\n Pets \n\n \n  <:col :let={pet} label=\"Name\" field={:name}><%= pet.name %></:col>\n  <:col :let={pet} label=\"Age\" field={:age}><%= pet.age %></:col>\n \n\n \n```\n\nIn this context, path points to the current route, and Flop Phoenix appends\npagination, filtering, and sorting parameters to it. You can use verified\nroutes, route helpers, or custom path builder functions. You'll find\nexplanations for the different formats in the documentation for\n`Flop.Phoenix.build_path/3`.\n\nNote that the field attribute in the `:col` slot is optional. If set and the\ncorresponding field in the schema is defined as sortable, the table header for\nthat column will be interactive, allowing users to sort by that column. However,\nif the field isn't defined as sortable, or if the field attribute is omitted, or\nset to `nil` or `false`, the table header will not be clickable.\n\nBy using the `for` option in your Flop query, Flop Phoenix can identify which\ntable columns are sortable. Additionally, it omits the `order` and `page_size`\nparameters if they align with the default values specified via `Flop.Schema`.\n\nYou also have the option to pass a `Phoenix.LiveView.JS` command instead of or\nin addition to a path. For more details, please refer to the component\ndocumentation.\n\nIf you wish to implement cursor-based pagination, see\n`Flop.Phoenix.cursor_pagination/1` for setup instructions.","ref":"readme.html#sortable-tables-and-pagination"},{"type":"extras","title":"Filter forms - Flop Phoenix","doc":"Flop Phoenix implements the `Phoenix.HTML.FormData` for the `Flop.Meta` struct.\nAs such, you can easily pass the struct to Phoenix form functions. One\nstraightforward way to render a filter form is through the\n`Flop.Phoenix.filter_fields/1` component, as shown below:\n\n```elixir\nattr :fields, :list, required: true\nattr :meta, Flop.Meta, required: true\nattr :id, :string, default: nil\nattr :on_change, :string, default: \"update-filter\"\nattr :target, :string, default: nil\n\ndef filter_form(%{meta: meta} = assigns) do\n  assigns = assign(assigns, form: Phoenix.Component.to_form(meta), meta: nil)\n\n  ~H\"\"\"\n  <.form\n    for={@form}\n    id={@id}\n    phx-target={@target}\n    phx-change={@on_change}\n    phx-submit={@on_change}\n  >\n    <.filter_fields :let={i} form={@form} fields={@fields}>\n      <.input\n        field={i.field}\n        label={i.label}\n        type={i.type}\n        phx-debounce={120}\n        {i.rest}\n      />\n    </.filter_fields>\n\n     reset \n  </.form>\n  \"\"\"\nend\n```\n\nNote that while the `filter_fields` component produces all necessary hidden\ninputs, it doesn't automatically render inputs for filter values. Instead, it\npasses the necessary details to the inner block, allowing you to customize the\nfilter inputs with your custom input component.\n\nYou can pass additional options for each field. Refer to the\n`Flop.Phoenix.filter_fields/1` documentation for details.\n\nNow, you can render a filter form like so:\n\n```elixir\n<.filter_form\n  fields={[:name, :email]}\n  meta={@meta}\n  id=\"user-filter-form\"\n/>\n```\n\nYou will need to handle the `update-filter` event with the `handle_event/3`\ncallback function of your LiveView.\n\n```elixir\n@impl true\ndef handle_event(\"update-filter\", params, socket) do\n  params = Map.delete(params, \"_target\")\n  {:noreply, push_patch(socket, to: ~p\"/pets?#{params}\")}\nend\n```","ref":"readme.html#filter-forms"},{"type":"extras","title":"LiveView streams - Flop Phoenix","doc":"To use LiveView streams, you can change your `handle_params/3` function as\nfollows:\n\n```elixir\ndef handle_params(params, _, socket) do\n  case Pets.list_pets(params) do\n    {:ok, {pets, meta}} ->\n      {:noreply,\n         socket\n         |> assign(:meta, meta)\n         |> stream(:pets, pets, reset: true)}\n\n    # ...\n  end\nend\n```\n\nWhen using LiveView streams, the data being passed to the table component\ndiffers. Instead of passing `@pets`, you'll need to use `@streams.pets`.\n\nThe stream values are tuples, with the DOM ID as the first element and the items\n(in this case, Pets) as the second element. You need to match on these tuples\nwithin the `:let` attributes of the table component.\n\n```elixir\n \n  <:col :let={{_, pet}} label=\"Name\" field={:name}><%= pet.name %></:col>\n  <:col :let={{_, pet}} label=\"Age\" field={:age}><%= pet.age %></:col>\n \n```","ref":"readme.html#liveview-streams"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"## [0.22.5] - 2023-12-24\n\n#","ref":"changelog.html#unreleased"},{"type":"extras","title":"Changed - Changelog","doc":"- Requires `phoenix_html ~> 4.0`.\n\n## [0.22.4] - 2023-11-18\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Don't render `li` element if a pagination link is not rendered.\n\n## [0.22.3] - 2023-11-14\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop ~> 0.24.0.\n\n## [0.22.2] - 2023-10-19\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Numbered pagination links were not wrapped in `li` elements.\n\n## [0.22.1] - 2023-09-28\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Allow to use `t:Phoenix.HTML.safe/0` as a label attribute in table headers.\n\n## [0.22.0] - 2023-09-26\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `directions` attribute to the `col` slot of the table component. This\n  allows you to override the default sort directions, e.g. to specify\n  nulls-first or nulls-last.\n- Added `thead_th_attrs` and `th_wrapper_attrs` attributes to the `col` slot\n  of the table component.\n- Added `thead_th_attrs` attribute to the `action` slot of the table component.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Renamed `attrs` attribute on the `col` and `action` slots of the table\n  component to `tbody_td_attrs` in order to match the naming of the global\n  table options.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Upgrade guide - Changelog","doc":"Rename the `attrs` attribute to `tbody_td_attrs` in both the `col` slot and the\n`action` slot:\n\n```diff\n \n-  <:col :let={p} attrs={[class=\"my-class\"]}><%= p.id %></:col>\n-  <:action :let={p} attrs={[class=\"my-class\"]}>button</:col>\n+  <:col :let={p} tbody_td_attrs={[class=\"my-class\"]}><%= p.id %></:col>\n+  <:action :let={p} tbody_td_attrs={[class=\"my-class\"]}>button</:col>\n \n```\n\n## [0.21.2] - 2023-09-26\n\n#","ref":"changelog.html#upgrade-guide"},{"type":"extras","title":"Changed - Changelog","doc":"- Support `phoenix_liveview ~> 0.20`.\n\n## [0.21.1] - 2023-08-05\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Allow passing a function to the `attrs` option of table component's `:action`\n  slot. Before, this was only supported in the `:col` slot.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Improve some error messages and documentation examples.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- In the `:col` and `:action` slots of the table component, the `attrs` option\n  did not properly override the attributes set with the `:tbody_td_attrs`\n  option.\n\n## [0.21.0] - 2023-07-17\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Depend on flop ~> 0.22.0.\n\n## [0.20.0] - 2023-07-09\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Added an `on_paginate` attribute to the `pagination` and `cursor_pagination`\n  components. This attribute takes a `Phoenix.LiveView.JS` command as a value.\n  The attribute can be combined with the `path` attribute, in which case the URL\n  will be patched _and_ the the given JS command is executed.\n- Similarly, an `on_sort` attribute was added to the `table` component.\n- Allow setting `tbody_tr_attrs` to a function that takes the current row item\n  as an argument and returns a list of attributes.\n- Allow setting the `attrs` attribute of the `:col` slot of the table component\n  to a function that takes the current row item as an argument and returns a\n  list of attributes.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The table ID falls back to `\"sortable-table\"` if no schema module is used.\n- The tbody ID was changed to `{id}-tbody`.\n- The table container ID is set to `{id}-container` now.\n- The table ID is set to `{id}` now.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- The `show` and `hide` attributes of the `:col` slot of the table component are\n  now deprecated in favor of the `:if` attribute.\n- The `event` attribute of the pagination, cursor pagination and table\n  components has been deprecated in favor of `on_pagination` and `on_sort`.\n\n## [0.19.1] - 2023-06-30\n\n#","ref":"changelog.html#deprecated"},{"type":"extras","title":"Changed - Changelog","doc":"- The table component only renders an ascending/descending order indicator in\n  the column header for the first order field, instead rendering one in the\n  column headers for all ordered fields.\n- Support Flop 0.22.\n\n## [0.19.0] - 2023-05-30\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Necessary updates for `phoenix_live_view ~> 0.19.0`.\n- Requires `phoenix_live_view ~> 0.19.0`.\n- Remove previously deprecated `Flop.Phoenix.pop_filter/2`. Use\n  `Flop.Filter.pop/3` instead.\n\n## [0.18.2] - 2023-05-08\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added `hidden` attribute to hidden inputs rendered by `filter_fields`\n  component in order to solve CSS spacing issues.\n\n## [0.18.1] - 2023-03-18\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added support for LiveView streams to the table component. To this end,\n  `row_id`, `row_item` and `id` attributes were added to the component,\n  following the example of Phoenix 1.7. The `id` attribute is added to the\n  `tbody`. If no `id` is explicitly set, a default value will be used depending\n  on the schema name.\n\n## [0.18.0] - 2023-02-25\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added dependency on `Phoenix.HTML` `~> 3.3.0`.\n- The `filter_fields` component now passes the `Phoenix.HTML.FormField` struct\n  introduced in `Phoenix.HTML` 3.3.0 to the inner block.\n- Support `:as` option for filter inputs with `Phoenix.HTML.FormData/4`\n  (`inputs_for`).\n- `Phoenix.HTML.FormData.input_type/2` now considers the Ecto type for join,\n  custom and compound fields.\n- Remove support for `path_helper` assigns, previously deprecated in 0.15.\n- Deprecate `Flop.Phoenix.pop_filter/2`.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"If your `input` component already knows how to handle the\n`Phoenix.HTML.FormField` struct, you can update the inner block for\n`filter_fields` like this:\n\n```diff\n<.filter_fields :let={i} form={f} fields={[:email, :name]}>\n  <.input\n-    field={{i.form, i.field}}\n+    field={i.field}\n    type={i.type}\n    label={i.label}\n-    id={i.id}\n-    name={i.name}\n-    value={i.value}\n    {i.rest}\n  />\n</.filter_fields>\n```\n\nIf your `input` component still expects the individual assigns, you can update\nthe inner block like this:\n\n```diff\n<.filter_fields :let={i} form={f} fields={[:email, :name]}>\n  <.input\n-    field={{i.form, i.field}}\n+    field={{i.field.form, i.field.field}}\n    type={i.type}\n    label={i.label}\n-    id={i.id}\n-    name={i.name}\n-    value={i.value}\n+    id={i.field.id}\n+    name={i.field.name}\n+    value={i.field.value}\n    {i.rest}\n  />\n</.filter_fields>\n```\n\nFor an upgrade example for the `path_helper` assign, see the changelog entry for\nversion 0.15.0.\n\n## [0.17.2] - 2023-01-15\n\n#","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop 0.19.\n\n## [0.17.1] - 2022-11-15\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Allow passing an `offset` when generating filter inputs with\n  `Phoenix.HTML.Form.inputs_for/3`.\n\n## [0.17.0] - 2022-10-27\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- New option `tbody_attrs` for table component.\n- New attribute `row_click` and slot `action` for table component.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- To pass additional attributes to a column, you will now have to use the\n  `attrs` attribute. This was necessary because defining a `global` attribute on\n  a slot causes a compile-time error in `phoenix_live_view` 0.18.3.\n\n```diff\n \n-  <:col :let={p} class=\"my-class\"><%= p.id %></:col>\n+  <:col :let={p} attrs={[class=\"my-class\"]}><%= p.id %></:col>\n \n```\n\n## [0.16.0] - 2022-10-10\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New Phoenix component `Flop.Phoenix.hidden_inputs_for_filter/1`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Major refactoring of `Flop.Phoenix.filter_fields/1`. Instead of giving you the\n  rendered ` ` and ` ` elements, the component now only passes the\n  necessary arguments to the inner block. You will have to pass these arguments\n  to your own `input` component (or whatever you name it). The field option\n  format has also been updated. These changes were made to fix warnings emitted\n  by live view 0.18.1, and also accompany current changes in Phoenix that thin\n  out `Phoenix.HTML` / `Phoenix.HTML.Form` in favor of Phoenix components for\n  inputs.\n- Require `flop >= 0.17.1 and < 0.19.0`.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed `Flop.Phoenix.filter_hidden_inputs_for/1`. This function is not used\n  internally anymore. You can either use `Phoenix.HTML.Form.hidden_inputs_for/1`\n  (Phoenix.HTML ~> 3.2), or use `Flop.Phoenix.hidden_inputs_for_filter/1`,\n  which does the same, but as a Phoenix component.\n- Removed `Flop.Phoenix.filter_label/1` and `Flop.Phoenix.filter_input/1`. With\n  the changes to `Flop.Phoenix.filter_fields/1` and the move away from the\n  input rendering functions of `Phoenix.HTML.Form`, these functions don't have\n  any value anymore. Read the documentation of\n  `Flop.Phoenix.hidden_inputs_for_filter/1` for an example on how to easily\n  render the fields of individual filters.\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed warnings about tainted variables in live view 0.18.1.\n- Fixed an issue where default parameters set in the backend module were not\n  removed from the query parameters.\n- Fixed URLs ending with `?` when no query parameters are necessary if the path\n  is passed as a string.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"##","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Filter fields component - Changelog","doc":"Previously, you would render a filter form like this:\n\n```elixir\n<.form :let={f} for={@meta}>\n   \n     \n      <%= entry.label %>\n      <%= entry.input %>\n     \n   \n</.form>\n```\n\nIn this example, `entry.label` and `entry.input` are complete ` ` and\n` ` elements with all attributes set. You will need to change this to:\n\n```elixir\n<.form :let={f} for={@meta}>\n  <.filter_fields :let={i} form={f} fields={[:name, :email]}>\n    <.input\n      id={i.id}\n      name={i.name}\n      label={i.label}\n      type={i.type}\n      value={i.value}\n      field={{i.form, i.field}}\n      {i.rest}\n    />\n  </.filter_fields>\n</.form>\n```\n\nYou will have to define an `input` component in your project. You can take a\nhint from the `input` component that is generated as part of the [Components\nmodule](https://github.com/phoenixframework/phoenix/blob/master/priv/templates/phx.gen.live/components.ex) by Phoenix 1.7.\n\n##","ref":"changelog.html#filter-fields-component"},{"type":"extras","title":"Field options - Changelog","doc":"Remove `input_opts` and `label_opts` and pass them directly to your `input`\ncomponent, or add them directly to the `input` component. If you passed an `id`\nto `filter_fields`, set in on the `form` instead.\n\n```diff\n<.filter_fields\n  :let={i}\n  form={f}\n  fields={[:name]}\n-  id=\"some-id\"\n-  input_opts={[class: \"input\", phx_debounce: 100]}\n-  label_opts={[class: \"label\"]}\n>\n  <.input\n     ...\n+    class=\"input\"\n+    phx-debounce={100}\n  />\n</.filter_fields>\n```\n\nUse strings instead of atoms to set the type, and use the types that your\n`input` component understands.\n\n```diff\n<.filter_fields\n  :let={i}\n  form={f}\n  fields={[\n-    name: [type: :text_input],\n+    name: [type: \"text\"],\n-    age: [type: :number_input],\n+    age: [type: \"number\"],\n-    phone: [type: :telephone_input],\n+    phone: [type: \"tel\"]\n  ]}\n>\n```\n\nIf you passed additional input function options in a tuple, take them out of\nthe tuple and add them to the keyword list instead.\n\n```diff\n<.filter_fields\n  :let={i}\n  form={f}\n  fields={[\n-    role: [type: {:select, [\"author\", \"editor\"], class: \"select\"}]\n+    role: [type: \"select\", options: [\"author\", \"editor\"], class: \"select\"]\n  ]}\n>\n```\n\nThe `default` option is not handled for you anymore. You can still set it, but\nit will just be passed on as part of the `rest` options, so your `input`\ncomponent will need to handle it.\n\n##","ref":"changelog.html#field-options"},{"type":"extras","title":"Filter label and input components - Changelog","doc":"If you used `Flop.Phoenix.filter_label/1` or `Flop.Phoenix.filter_input/1`\nbefore, follow the example in the documentation of\n`Flop.hidden_inputs_for_filter/1` to render the inputs of individual filters\nwithout the removed components.\n\n## [0.15.2] - 2022-10-10\n\n#","ref":"changelog.html#filter-label-and-input-components"},{"type":"extras","title":"Changed - Changelog","doc":"- Change version requirement for Flop to `>= 0.15.0 and < 0.19.0`.\n\n## [0.15.1] - 2022-09-30\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Typespec of `Flop.Phoenix.build_path/3`.\n\n## [0.15.0] - 2022-09-22\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"This release adds support for passing URI strings instead of MFA or FA tuples to\ncomponents and helper functions. This allows you to use the library with the\nverified routes introduced in Phoenix 1.7. Alternatively, you can now also\nchoose to pass a 1-ary path builder function. See `Flop.Phoenix.build_path/3`\nfor examples. Passing tuples pointing to route helpers is still supported.\n\n- Added an example for a custom filter form component to the readme.\n- Support passing a URI string as a path to the `table`, `pagination` and\n  `cursor_pagination` components and to `build_path/3`.\n- Support passing a 1-ary path builder function to the `table`, `pagination` and\n  `cursor_pagination` components and to `build_path/3`.\n- New function `Flop.Phoenix.pop_filter/2`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Require `live_view ~> 0.18.0`.\n- Deprecate `path_helper` assign in favor of `path`.\n- Use declarative assigns and replace `Phoenix.LiveView.Helpers.live_patch/1`\n  with `Phoenix.Component.link/1`.\n- `Flop.Phoenix.filter_input/1` requires additional options for the input\n  function to be passed in the `input_opts` assign, instead of passing them\n  directly to the component. This was necessary because the global attributes\n  you can define with declarative assigns in LiveView 0.18 are meant for HTML\n  attributes, while the input options may contain any additional attributes\n  necessary (e.g. a list of select options that are rendered as option\n  elements).\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Apply `show` and `hide` attribute for columns to `colgroup` as well.\n- Correctly handle multiple inputs for the same field in `Flop.filter_fields/1`.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"Rename the `path_helper` assigns of `table`, `pagination` and\n`cursor_pagination` components to `path`.\n\n```diff\n- <.pagination meta={@meta} path_helper={{Routes, :pet_path, [@socket, :index]}} />\n+ <.pagination meta={@meta} path={{Routes, :pet_path, [@socket, :index]}} />\n```\n\nWrap additional options passed to `Flop.Phoenix.filter_input/1` into a single\n`input_opts` assign.\n\n```diff\n- <.filter_input form={ff} class=\"select\" options={[:some, :options]} />\n+ <.filter_input form={ff} input_opts={[class: \"select\", options: [:some, :options]]} />\n```\n\n## [0.14.2] - 2022-08-26\n\n#","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop `~> 0.17.0`.\n\n## [0.14.1] - 2022-03-22\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop `~> 0.16.0`.\n\n## [0.14.0] - 2022-02-22\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `symbol_unsorted` option for the `table` component.\n- `caption` assign for the `table` component.\n- `col_style` assign for the `:col` slot of the `table` component.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Additional attributes passed to the `<:col>` slot will now be added as\n  attributes to the ` ` tags.\n\n## [0.13.0] - 2021-11-14\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `cursor_pagination/1` component.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The pagination component adds the `disabled` class to the `span` that is\n  displayed when the previous or next link is disabled now. Previously, the\n  `disabled` attribute was set on the `span`. The class can be customized with\n  the `:disabled_class` option.\n\n## [0.12.0] - 2021-11-08\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Implement the `Phoenix.HTML.FormData` protocol for `Flop.Meta`. This means\n  you can pass the meta struct as `:for` option to the Phoenix `form_for`\n  functions now.\n- Add the functions `filter_fields/1`, `filter_input/1` and `filter_label/1`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove `:for` option. The schema module is now automatically derived from the\n  meta struct.\n\n## [0.11.1] - 2021-10-31\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `hide` and `show` options to table `:col`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Passing a `label` to a table `:col` is now optional.\n\n## [0.11.0] - 2021-10-30\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- The `path_helper_args` assign has been removed in favor of passing mfa\n  tuples as `path_helper`.\n- In the same vein, `Flop.Phoenix.build_path/4` has been replaced with\n  `Flop.Phoenix.build_path/3`, which also takes a tuple as the first argument.\n- The table component has been changed to use slots. The `headers`,\n  `footer`, `row_func` and `row_opts` assigns have been removed. Also, the\n  `tfoot_td_attrs` and `tfoot_th_attrs` options have been removed.\n- The `live_view` version requirement has been changed to `~> 0.17.0`.\n- Better error messages for invalid assigns have been added.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"Update the `path_helper` and `path_helper_args` assigns set for the `table`\nand `pagination` component:\n\n```diff\n- path_helper={&Routes.pet_path/3}\n- path_helper_args={[@socket, :index]}\n+ path_helper={{Routes, :pet_path, [@socket, :index]}}\n```\n\nIf you prefer, you can pass a function instead.\n\n```diff\n+ path_helper={{&Routes.pet_path/3, [@socket, :index]}}\n```\n\nUpdate any calls to `Flop.Phoenix.build_path/4`:\n\n```diff\n- Flop.Phoenix.build_path(&Routes.pet_path/3, [@socket, :index], meta)\n+ Flop.Phoenix.build_path({Routes, :pet_path, [@socket, :index]}, meta)\n```\n\nIf you prefer, you can use a 2-tuple here as well:\n\n```diff\n+ Flop.Phoenix.build_path({&Routes.pet_path/3, [@socket, :index]}, meta)\n```\n\nFinally, update the tables in your templates:\n\n```diff\n  [pet.name, pet.age] end}\n-   footer={[\"\", @average_age]}\n- />\n+ >\n+   <:col let={pet} label=\"Name\" field={:name}><%= pet.name %></:col>\n+   <:col let={pet} label=\"Age\" field={:age}><%= pet.age %></:col>\n\n+   <:foot>\n+      \n+         \n+        <%= @average_age %> \n+      \n+   </:foot>\n+  \n```\n\nAlso, you can remove `tfoot_td_attrs` and `tfoot_th_attrs` from the `opts`\nassign (or opts provider function).\n\n## [0.10.0] - 2021-10-24\n\n#","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Added - Changelog","doc":"- It is now possible to set global options for the components in your config.\n\n```elixir\nconfig :flop_phoenix,\n  pagination: [opts: {MyApp.ViewHelpers, :pagination_opts}],\n  table: [opts: {MyApp.ViewHelpers, :table_opts}]\n```\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The `for`, `event` and `target` options moved from the `opts` assign to the\n  root. The `opts` assign is now exclusively used for customization options\n  that modify the appearance, which are usually set globally for a\n  project and are not related to the specific data or view.\n- The `row_func/2` function passed to the `table` component receives the new\n  `row_opts` assign now instead of the `opts` assign.\n- The pagination and table components only pass the `for` option to the query\n  builder, instead of all `opts`.\n- The `path_helper` and `path_helper_args` assigns are now optional if an\n  `event` is passed. A descriptive error is raised if neither of them are\n  passed.\n- The `opts` assign for the pagination and table components is now optional.\n- Aria labels were added to the links to the first and last page.\n- The `aria-sort` attribute was added to the table headers.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"1. Remove the `for`, `event` and `target` from the `opts` assign and add them\n   as regular assigns at the root level.\n2. Move any key/value pairs that are needed by your `row_func` from `opts` to\n   `row_opts`.\n\nFor example, if your `row_func` looks like this:\n\n```elixir\ndef table_row(%Pet{id: id, name: name}, opts) do\n  socket = Keyword.fetch!(opts, :socket)\n  [name, link(\"show\", to: Routes.pet_path(socket, :show, id))]\nend\n```\n\nUpdate your template like this:\n\n```diff\n \n\n \n```\n\n## [0.9.1] - 2021-10-22\n\n#","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Change `live_view` version requirement to `~> 0.16.0 or ~> 0.17.0`.\n\n## [0.9.0] - 2021-10-04\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add table foot option for sortable table.\n\n## [0.8.1] - 2021-08-11\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Loosen version requirement for Flop.\n\n## [0.8.0] - 2021-08-11\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New options `event` and `target` for the pagination and sortable table\n  component, which allow to emit pagination and sorting events in LiveView\n  without patching the URL.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Use `HEEx` templates for both the pagination and the sortable table component.\n  Refer to the Readme for usage examples.\n- Require `live_view ~> 0.16.0`.\n- Support safe HTML tuples in unsortable table headers.\n- Improve documentation with examples for LiveView, HEEx templates and EEx\n  templates.\n\n## [0.7.0] - 2021-06-13\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add wrapper around sortable table header link and order direction indicator.\n- Add option `current_link_attrs` to pagination builder.\n- Add options `thead_tr_attrs`, `thead_th_attrs`, `tbody_tr_attrs` and\n  `tbody_td_attrs` to table generator.\n- Add option `no_results_content` to table generator, which sets the content\n  that is going to be displayed instead of the table if the item list is empty.\n  A default option is applied, so make sure to set the option and/or remove your\n  own no results messages from your templates when making the upgrade.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The table options `table_class`, `th_wrapper_class`, `symbol_class` and\n  `container_class` were replaced in favour of `table_attrs`,\n  `th_wrapper_attrs`, `symbol_attrs` and `container_attrs` for more flexibility\n  and consistency with the pagination generator. To update, rename the options\n  in your code and wrap the values in keyword lists with a `class` key\n  (e.g. `container_class: \"table-container\"` =>\n  `container_attrs: [class: \"table-container\"]`).\n- The `pagination_link_attrs` is not applied to current links anymore. Use\n  `current_link_attrs` to set the attributes for the current link.\n- Omit `page=1` and `offset=0` when building query parameters.\n- Omit default values for order and limit/page size parameters when building\n  query parameters.\n- Requires Flop `~> 0.11.0`.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Order direction indicator was wrapped twice.\n- A Flop struct with an offset was resulting in invalid pagination links.\n\n## [0.6.1] - 2021-05-05\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Pagination helper generated invalid links when using `default_limit` option.\n\n## [0.6.0] - 2021-05-04\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Flop.Phoenix.table/1` for rendering sortable tables.\n- Add function `Flop.Phoenix.to_query/1`, which converts a Flop struct into\n  a keyword list for query parameters.\n- Add function `Flop.Phoenix.build_path/3`, which applies Flop parameters to a\n  Phoenix path helper function.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Remove `Flop.Phoenix.Live.PaginationComponent` in favor of making\n  `Flop.Phoenix.pagination/4` work in both `.eex` and `.leex` templates.\n\n## [0.5.1] - 2021-04-14\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Merge pagination query parameters into existing query parameters, if present.\n\n## [0.5.0] - 2021-03-23\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Rename `FlopPhoenix` to `Flop.Phoenix`.\n- Add `Flop.Phoenix.Live.PaginationComponent` for use with `Phoenix.LiveView`.\n- Change pagination to always display links to the first and last page.\n\n## [0.4.0] - 2020-09-04\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Allow usage with newer versions of Flop.\n\n## [0.3.0] - 2020-06-17\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New option to hide the number of page links.\n- New option to limit the number of page links.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Add order and filter parameters to pagination links.\n\n## [0.2.0] - 2020-06-15\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Improve documentation.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `previous_link/3`, `next_link/3` and `page_links/3` are private functions now.\n\n## [0.1.0] - 2020-06-15\n\nInitial release","ref":"changelog.html#changed"}],"content_type":"text/markdown"}